#include "i2c.h"
#define DS1307_ID 0xD0	
#define SEC   0x00
#define	MIN   0x01	
#define HOUR  0x02
#define DAY	  0x03
#define DATE  0x04
#define MONTH 0x05
#define YEAR  0x06

sbit UP=P1^0;
sbit DOWN=P1^1;
sbit SET=P1^2;
sbit RST=P1^3;
sbit TSET=P1^5;
sbit BUZZ=P2^6;
unsigned char bell_time[3];
unsigned char current_time[3];
unsigned char alrm,alrmp;
code const unsigned char * MonthStr[12]={{"Jan"},
										{"Feb"},
										{"Mar"},
										{"Apr"},
										{"May"},
										{"Jun"},
										{"Jul"},
										{"Aug"},
										{"Sep"},
										{"Oct"},
										{"Nov"},
										{"Dec"}};
code const unsigned char * DayStr[7]  ={{"Sun"},
				   						{"Mon"},
				   						{"Tue"},
				   						{"Wen"},
				   						{"Thu"},
				   						{"Fri"},
				   						{"Sat"}};

unsigned char DS1307_get(unsigned char addr)
{
	unsigned char ret;	
	I2C_start();            /* Start i2c bus */
	I2C_write(DS1307_ID);   /* Connect to DS1307 */
	I2C_write(addr);		/* Request RAM address on DS1307 */
	I2C_start();			/* Start i2c bus */
	I2C_write(DS1307_ID+1);	/* Connect to DS1307 for Read */
	ret = I2C_read();		/* Receive data */
	I2C_stop();				/* Stop i2c bus */
	return ret;
 
}

void DS1307_settime(unsigned char dy,unsigned char hh, unsigned char mm, unsigned char ss)
{
	I2C_start(); 

	I2C_write(DS1307_ID);	/* connect to DS1307 */ 
	I2C_write(0x00);		/* Request RAM address at 00H */ 
   	I2C_write(ss);			/* Write sec on RAM address 00H */
	I2C_write(mm);			/* Write min on RAM address 01H */
	I2C_write(hh);
	I2C_write(dy);				/* Write hour on RAM address 02H */
   	I2C_stop();           	/* Stop i2c bus */
}

void DS1307_setdate(unsigned char dd, unsigned char mm, unsigned char yy)
{
 	I2C_start();	
  	I2C_write(DS1307_ID);	/* connect to DS1307 */
	I2C_write(0x04);		/* Request RAM address at 04H */ 	
	I2C_write(dd);			/* Write date on RAM address 04H */
	I2C_write(mm);			/* Write month on RAM address 05H */
	I2C_write(yy);			/* Write year on RAM address 06H */
	I2C_stop();				/* Stop i2c bus */
}

void send2lcd(unsigned char value)
{
	unsigned char buf = 0;	
	
	buf = value & 0xF0;		/* Filter for high byte */
	buf = (buf>>4)|(0x30);		/* Convert  to ascii code */

	WriteCharLCD(buf);			/* Show on LCD */

	buf = value & 0x0F;		/* Filter for low byte */
	buf = buf | 0x30;       	/* Convert to ascii code */

	WriteCharLCD(buf);			/* Show on LCD */
}

char * Int2Month(unsigned char month)
{
  return MonthStr[(month)-1];
}
unsigned char BCD2HEX(unsigned char bcd)
{
	
	return( ((bcd >> 4)*10) + (bcd & 0x0f) );
}
unsigned char INT2BCD(unsigned char x)
{
	unsigned char temp=0;
	temp=x%10;
	x/=10;
	temp=(x<<4)|temp;
	return temp;		
	
}
char * Int2Day(unsigned char day)
{
	return DayStr[day-1];
}
void dispDateTime(void)
{
 	unsigned char ss,mm,hh,dy,dd,mh,yy;
	ss = DS1307_get(SEC);
	mm = DS1307_get(MIN);
	hh = DS1307_get(HOUR);
	dy = DS1307_get(DAY);
	dd = DS1307_get(DATE);
	mh = DS1307_get(MONTH);
	yy = DS1307_get(YEAR);
	DisplayStrLCD("Date:");
	send2lcd(dd);//display date 
	WriteCharLCD('-');
	DisplayStrLCD(Int2Month(BCD2HEX(mh)));	/* Show month on LCD */
	WriteCharLCD('-');
	DisplayStrLCD("20");
	send2lcd(yy);		/* Show year on LCD */
	GoToLCD(2,1);
	DisplayStrLCD("Time:");
	send2lcd(hh);
	WriteCharLCD(':');
	send2lcd(mm);
}
void SetTime(void)
{
 	unsigned char min,hr,day,dat,mon,yr,set;
	ClearScreenLCD();
	DelayMS(50);
	GoToLCD(1,1);
	DisplayStrLCD("Enter Minutes:");
	GoToLCD(2,1);
	set=BCD2HEX(DS1307_get(MIN));//set minutes
	while(SET==1)
	{
		GoToLCD(2,9);
		if(set<10)
		{
			WriteCharLCD('0');	
		}
		if(set>59 || set<0)
		{
			set=0;
		}
		DisplayIntegerLCD(set);
		if(UP==0)
		{	if(set==59)
				set=0;
			else
				set++;
			DelayMS(200);
		}
		if(DOWN==0)
		{
			
			if(set==0)
				set=59;
			else
				set--;
			DelayMS(200);
		}
		
		if(SET==0)
		{
			break;
		}
	}
	min=INT2BCD(set);
	//set hours
	GoToLCD(1,1);
	SET=UP=DOWN=1;
	ClearScreenLCD();
	DelayMS(50);
	DisplayStrLCD("Enter Hours:");
	set=BCD2HEX(DS1307_get(HOUR));
	
	{
		GoToLCD(2,1);
		while(SET==1)
		{
			GoToLCD(2,1);
			if(set<10)
			{
				WriteCharLCD('0');	
				
			}
			if(set>23 || set<0)
			{
				set=0;
			}
			DisplayIntegerLCD(set);
			if(UP==0)
			{	
				if(set==23)
					set=0;	
				else
					set++;
				DelayMS(200);
			}
			if(DOWN==0)
			{
				if(set==0)
					set=23;
				else
					set--;
				DelayMS(200);
			}
		}
		hr=INT2BCD(set);
	}
	
	ClearScreenLCD();
	DelayMS(200);
	GoToLCD(1,1);
	DisplayStrLCD("Enter Day:");
	set=BCD2HEX(DS1307_get(DAY));
	while(SET==1)
	{
		GoToLCD(2,1);
		if(set>7 || set<1)
		{
			set=1;
		}
		DisplayStrLCD(Int2Day(set));
		if(UP==0)
		{
			set++;
			DelayMS(200);
		}
		if(DOWN==0)
		{
			
			if(set==1)
				set=7;
			else
				set--;
			
			DelayMS(200);
		}
	}
	day = INT2BCD(set);
	ClearScreenLCD();
	//set date
	DelayMS(200);

	GoToLCD(1,1);
	DisplayStrLCD("Enter Date:");
	set=BCD2HEX(DS1307_get(DATE));
	while(SET==1)
	{
		GoToLCD(2,1);
		if(set<10)
		{
			WriteCharLCD('0');	
			
		}
		if(set>31 || set<0)
		{
			set=1;
		}
		DisplayIntegerLCD(set);
		if(UP==0)
		{
			set++;
			DelayMS(200);
		}
		if(DOWN==0)
		{
			
			if(set==0)
				set=31;
			else
				set--;
			DelayMS(200);
		}
		
		
	}
	dat= INT2BCD(set);
	ClearScreenLCD();
	DelayMS(200);
	GoToLCD(1,1);
	DisplayStrLCD("Enter Month:");
	set=BCD2HEX(DS1307_get(MONTH));
	while(SET==1)
	{
		GoToLCD(2,1);
		if(set>12 || set<=0)
		{
			set=1;
		}
		if(set==0)
				set=12;
		DisplayStrLCD(Int2Month(set));
		if(UP==0)
		{
			set++;
			DelayMS(200);
		}
		if(DOWN==0)
		{
			
			if(set==1)
				set=12;
			else
				set--;
			DelayMS(200);
		}
		
		
	}
	mon = INT2BCD(set);
	ClearScreenLCD();
	DelayMS(200);
	GoToLCD(1,1);
	DisplayStrLCD("Enter year:");
	GoToLCD(2,1);
	DisplayStrLCD("20");
	set=BCD2HEX(DS1307_get(YEAR));
	while(SET==1)
	{
		GoToLCD(2,3);
		if(set>99 || set<10)
		{
			set=10;
		}
		DisplayIntegerLCD(set);
		if(UP==0)
		{
			set++;
			DelayMS(200);
		}
		if(DOWN==0)
		{
			
			if(set==0)
				set=99;
			else
				set--;
			DelayMS(200);
		}
	}
	yr=INT2BCD(set);
	ClearScreenLCD();
	DelayMS(50);
   	DS1307_settime(day,hr,min,0x00);
	DS1307_setdate(dat,mon,yr);
	DisplayStrLCD("Saved Successfully");
	
}
void SetBellTime(unsigned char bellno)//this function stores data of a bell time
{
	unsigned char min,hr,set,loc;
	unsigned char dat[3];
	ClearScreenLCD();

	loc = bellno*2 - 2;
	RdFromROM(dat,loc,2);
	dat[2]=0;
	DelayMS(50);

	GoToLCD(1,1);
	DisplayStrLCD("No:");
	DisplayIntegerLCD(bellno);
	DisplayStrLCD("-Minutes:");
	
	
	GoToLCD(2,1);
	if(dat[0]<10)
	{
		WriteCharLCD('0');	
	}
	DisplayIntegerLCD(dat[0]);
	WriteCharLCD(':');
	GoToLCD(2,4);
	if(dat[1]<10)
	{
		WriteCharLCD('0');	
	}
	DisplayIntegerLCD(dat[1]);
	
	
	set=dat[1];//set minutes
	UP=TSET=DOWN=1;
	while(TSET==1)
	{
		GoToLCD(2,4);
		if(set<10)
		{
			WriteCharLCD('0');	
		}
		if(set>59 || set<0)
		{
			set=0;
		}
		DisplayIntegerLCD(set);
		if(UP==0)
		{	if(set==59)
				set=0;
			else
				set++;
			DelayMS(200);
		}
		if(DOWN==0)
		{
			
			if(set==0)
				set=59;
			else
				set--;
			DelayMS(200);
		}
		
		if(TSET==0)
		{
			break;
		}
	}
	min=set;
	GoToLCD(1,1);
	DisplayStrLCD("                ");
	TSET=1;
	//ClearScreenLCD();
	GoToLCD(1,1);
	DelayMS(250);
	DisplayStrLCD("No:");
	DisplayIntegerLCD(bellno);
	DisplayStrLCD("-Hours:");
	set=dat[0];
	GoToLCD(2,1);
	while(TSET==1)
	{
			GoToLCD(2,1);
			if(set<10)
			{
				WriteCharLCD('0');	
				
			}
			if(set>23 || set<0)
			{
				set=0;
			}
			DisplayIntegerLCD(set);
			WriteCharLCD(':');
			if(UP==0)
			{	
				if(set==23)
					set=0;	
				else
					set++;
				DelayMS(200);
			}
			if(DOWN==0)
			{
				if(set==0)
					set=23;
				else
					set--;
				DelayMS(200);
			}
	}
	hr=set;
	ClearScreenLCD();
   	DisplayStrLCD("Saving...");
	WrCharROM(hr,loc);
	DelayMS(50);
	WrCharROM(min,loc+1);
	ClearScreenLCD();
	DisplayStrLCD("Saved...");
	DelayMS(500);
	ClearScreenLCD();
}
void BellSetting(void)
{
	unsigned char n;
	DisplayStrLCD("Bell Settings:");
	DelayMS(3000);
	ClearScreenLCD();
	for(n=1;n<=25;n++)
	{
	 	SetBellTime(n);
	}
}
void GetBellTime(unsigned char bellno)
{
 	unsigned char loc;
	ClearScreenLCD();
	loc = bellno*2 - 2;
	RdFromROM(bell_time,loc,2);
	bell_time[2]=0;
}
void GetCurrentTime(void)
{
  	unsigned char mm,hh;
	mm = DS1307_get(MIN);
	hh = DS1307_get(HOUR);
	current_time[1] = BCD2HEX(mm);
	current_time[0] = BCD2HEX(hh);
}
void BuzzerOn(void)
{
 	unsigned char i,n,o;
	
	if(alrm!=alrmp)
	{
		for(n=0;n<200;n++)
		{
			for(i=0;i<150;i++)
			{
			 	BUZZ=0;
				for(o=0;o<50;o++);
				BUZZ=1;
			}
		}
		alrmp=alrm;
	}

	BUZZ=1;
	
}

void CheckBell(void)
{
 	unsigned char n;
	for(n=1;n<=25;n++)
	{
	 	GetBellTime(n);//Get Bell Time And compare with current time if same raise alarm.
		GetCurrentTime();
		if(current_time[1]==bell_time[1]) //compare hours
		{
			if(current_time[0]==bell_time[0])//compare minutes
			{
					alrm=n;
					BuzzerOn();
			}
			else
			{
					alrm=0;
			}
				
		}
		else
		{
			alrm=0;
		}
	}
}
void ResetAll(void)
{
	unsigned char loc,bellno;
	DisplayStrLCD("Erasing all data...");
	for(bellno=1;bellno<=25;bellno++)
	{
		loc = bellno*2 - 2;
		WrCharROM(0,loc); 	//write 00 
		DelayMS(50);
		WrCharROM(0,loc+1);	//write 00
		DelayMS(50);
	}
	ClearScreenLCD();
	DisplayStrLCD("Erased...");
	DelayMS(2500);
	ClearScreenLCD();
}